import datetime
import time
import PICommon_pb2
import PIDataScenario_pb2
import remoteapi

"""
TestLiveScenario.py

Unit test for Remote API that test commands that require a live scenario
"""
client = remoteapi.DiscoveryClient().createClient()

if client is None:
	raise Exception("Failed to connect.")

pi = client.pluginApi()
piData = pi.data()
piCommon = pi.common()

pi.config().setResponsesEnabled(True)

# A previous run could have crashed, so re-enable Replay to leave SIMDIS in a good state
pi.simdis().enableReplay(True, "Test Plug-in", "Making sure a plug-in can disable Replay")

# delete the scenario
f = piData.deleteScenario()
if not f.isOk(5000):
	raise Exception("Failed to delete scenario")

# create a new live scenario to test live-only commands
f = piData.initializeScenario("Unit test live mode", 0, 0, True)
if not f.isOk(1000):
	raise Exception("Failed to initialize live scenario")
currDateTime = datetime.datetime.utcnow()
startOfYear = datetime.datetime(currDateTime.year,1,1)
secsSinceYear = (currDateTime - startOfYear).total_seconds()
f = piData.setReferenceYear(currDateTime.year)
if not f.isOk(1000):
	raise Exception("Failed to set reference year")
piData.clockSetMode(PIDataScenario_pb2.CLOCKMODE_FREEWHEEL, secsSinceYear)
piData.clockSetTime(secsSinceYear)

# Create a platform to add live data points to
f = piData.createPlatform("Unit test live platform", "x", 1)
if not f.isOk(1000):
	raise Exception("Failed to create a new platform")
platId = f.value()

# Test start recording
f = piCommon.startRecording("$(TMP)/OkToDeleteThisFile.asi", True, False, 1)
if not f.isOk(1000):
	raise Exception("Failed to start recording")
if f.value() != PICommon_pb2.RECORDING_OK:
	raise Exception("startRecording() returned unexpected recording state: " + str(f.value()))

# Add data to the platform
for i in range(10):
	currDateTime = datetime.datetime.utcnow()
	startOfYear = datetime.datetime(currDateTime.year,1,1)
	secsSinceYear = (currDateTime - startOfYear).total_seconds()
	piData.addPlatformData(platId, secsSinceYear, 0, 0, 0)
	piData.addGenericData(platId, secsSinceYear, "Time", str(secsSinceYear))
	piData.addCategoryData(platId, secsSinceYear, "Time", str(secsSinceYear))
	time.sleep(1)

# Test stop recording
f = piCommon.stopRecording()
if not f.isOk(1000):
	raise Exception("Failed to stop recording")
if f.value() != PICommon_pb2.RECORDING_OK:
	raise Exception("stopRecording() returned unexpected recording state: " + str(f.value()))

# Replay should not be active
f = piData.clockIsVisualTimeActive();
if not f.isOk(1000):
	raise Exception("Failed to call clockIsVisualTimeActive")
if f.value() != False:
	raise Exception("clockIsVisualTimeActive()() Replay should not be active")

# Start Replay
f = piCommon.executeAction("Replay")
if not f.isOk(1000):
	raise Exception("Failed to enter Replay Mode")

print("Will do 10 seconds of Replay.")
time.sleep(10)

# Replay should be active
f = piData.clockIsVisualTimeActive();
if not f.isOk(1000):
	raise Exception("Failed to call clockIsVisualTimeActive")
if f.value() != True:
	raise Exception("clockIsVisualTimeActive()() Replay should be active")

# Set visual time to the start time
f = piData.clockGetBegin()
if not f.isOk(1000):
	raise Exception("Failed to get begin time")

f = piData.setVisualTime(f.value())
if not f.isOk(1000):
	raise Exception("Failed to set visual time")

# Should be a time difference
dataTime = piData.getTime().value()
visualTime = piData.getVisualTime().value()
delta = abs(dataTime - visualTime)
if delta < 1.0:
	print("Delta = %.2f" % delta)
	print("Data Time = %.2f" % dataTime)
	print("Visual Time = %.2f" % visualTime)
	raise Exception("Expecting a difference between the data time and the visual time.")

# Generic Data should be different
visualClockData = piData.currentGenericData(platId, False)
dataClockData = piData.currentGenericData(platId, True)
if visualClockData == dataClockData:
	raise Exception("currentGenericData() returned invalid results")

# Category Data should be different
visualClockData = piData.currentCategoryData(platId, False)
dataClockData = piData.currentCategoryData(platId, True)
if visualClockData == dataClockData:
	raise Exception("currentCategoryData() returned invalid results")

# Turn Replay off
print("The plug-in will turn off the Replay.  Click OK for the dialog displayed by SIMDIS.")
f = pi.simdis().enableReplay(False, "Test Plug-in", "Making sure a plug-in can disable Replay")
if not f.isOk(1000):
	raise Exception("Failed to disable Replay")

time.sleep(1)

# Re-enable Replay
f = pi.simdis().enableReplay(True, "Test Plug-in", "Making sure a plug-in can disable Replay")
if not f.isOk(1000):
	raise Exception("Failed to enable Replay")

# Replay is enabled but not on, so the two times should match
dataTime = piData.getTime().value()
visualTime = piData.getVisualTime().value()
delta = abs(dataTime - visualTime)
if delta > 1.0:
	print("Delta = %.2f" % delta)
	print("Data Time = %.2f" % dataTime)
	print("Visual Time = %.2f" % visualTime)
	raise Exception("Expecting no difference between the data time and the visual time.")

# Generic Data should be the same since the clocks now match
visualClockData = piData.currentGenericData(platId, False)
dataClockData = piData.currentGenericData(platId, True)
if visualClockData.value()[0].data != dataClockData.value()[0].data:
	raise Exception("currentGenericData() returned invalid results")

# Category Data should be the same since the clocks now match
visualClockData = piData.currentCategoryData(platId, False)
dataClockData = piData.currentCategoryData(platId, True)
if visualClockData.value()[0].value != dataClockData.value()[0].value:
	raise Exception("currentCategoryData() returned invalid results")

piData.clockSetMode(PIDataScenario_pb2.CLOCKMODE_STEP, piData.clockGetBegin().value())
piData.clockStop()

print("Test successful")
