from . import PICommonSender
from . import PIDataSender
from . import PIPlotXYSender
from . import PISIMDISSender
from . import ConfigurationSender
import remoteapi

###############################################################################################
# Provides a simple way for Plug-in API methods implemented in Python to provide values back to
# users.  Instantiate one of these in your ___Sender.py method and provide a functor for what
# you want your end user to retrieve, given a Response out of the value() call.  For example:
#
# def isScenarioInitialized(self):
#   ...
#   return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
#     lambda r: r.piData.scenario.isScenarioInitialized)
#
# End users can then call your method:
#
# f = pi.data().isScenarioInitialized()
# isInit = f.value(1000) # Wait 1000 ms for a response
#
class ResponseHandler(object):
	def __init__(self, client, statusAndSequenceId, getValueFunction):
		self.client_ = client
		self.status_ = statusAndSequenceId[0]
		self.sequenceId_ = statusAndSequenceId[1]
		self.getValueFunction_ = getValueFunction
		self.response_ = None
		self.expectingResponse_ = True
		self.responseStatus = remoteapi.CONNECTION_BLOCKED

	# Returns True if there is a response ready, even if that response is invalid
	def waitForResponse(self, waitMs=1000):
		if self.expectingResponse_:
			statResp = self.client_.getResponse(self.sequenceId_, waitMs)
			if statResp[0] == remoteapi.CONNECTION_BLOCKED:
				return False
			self.expectingResponse_ = False
			self.responseStatus = statResp[0]
			self.response_ = statResp[1]
		return True

	# Returns the protobuf-given value for the method; may be None. Uses functor from construction to retrieve value from response 
	def value(self, waitMs=1000):
		if not self.waitForResponse(waitMs):
			return None

		if self.response_ is None:
			return None
		if self.getValueFunction_ is None:
			return self.response_
		return self.getValueFunction_(self.response_)

	# Returns true if we got a response with a valid return code, optionally also waiting for that response
	def isOk(self, waitForResponseMs=0):
		self.waitForResponse(waitForResponseMs)
		return self.responseStatus == remoteapi.CONNECTION_OK and self.response_.response.returnCode == 0

	# Returns True if the send was successful
	def sent(self):
		return self.status_ is remoteapi.CONNECTION_OK

	# Returns True if we got some sort of response (including invalid/unexpected response)
	def responseReceived(self):
		return not self.expectingResponse_

	# Returns True if we got some sort of response (including invalid/unexpected response)
	def responseDescription(self):
		if self.expectingResponse_:
			return "No response received."
		return self.response_.response.description

###############################################################################################
# Encapsulates Plug-in API namespaces and Remote API configuration
class PluginApi(object):
	def __init__(self, client):
		self.client_ = client
		self.common_ = PICommonSender.PICommonSender(client)
		self.data_ = PIDataSender.PIDataSender(client)
		self.plotxy_ = PIPlotXYSender.PIPlotXYSender(client)
		self.simdis_ = PISIMDISSender.PISIMDISSender(client)
		self.configuration_ = ConfigurationSender.ConfigurationSender(client)
	def common(self):
		return self.common_
	def data(self):
		return self.data_
	def plotxy(self):
		return self.plotxy_
	def simdis(self):
		return self.simdis_
	def config(self):
		return self.configuration_
